;+
; Project     : HESSI
;
; Name        : EIT_COPY
;
; Purpose     : process & copy latest EIT FITS files to HESSI archive. 
;
; Category    : synoptic gbo
;
; Syntax      : IDL> eit_copy,tstart,tend,back=back,_extra=extra
;
; Inputs      : TSTART, TEND = start and end times to consider
;               [def = current day]
;
; Outputs     : EIT FITS files.
;
; Keywords    : BACK = days to look back
;               COUNT = # of files processed
;               CLOBBER = set to clobber existing files
;               PREFLARE = minutes of preflare data to include [def=5]
;               POSTFLARE = minutes of postflare data to include [def=10]
;               ALL_DAY = process all day
;
; Restrictions: Unix only
;
; History     : Written 14 Aug 2001, D. Zarro (EITI/GSFC)
;
; Contact     : dzarro@solar.stanford.edu
;-

;----------------------------------------------------------------------------

pro eit_copy,tstart,tend,count=count,back=back,verbose=verbose,$
             clobber=clobber,_extra=extra,out_dir=out_dir,$
             reprocess=reprocess,$
             preflare=preflare,postflare=postflare,all_day=all_day

if os_family(/lower) ne 'unix' then begin
 message,'Sorry, Unix only',/cont
 return
endif

clobber=keyword_set(clobber) or keyword_set(reprocess)
verbose=keyword_set(verbose)

;-- decode directory environments

if exist(out_dir) then begin
 if not test_dir(out_dir) then return
 synop_images=out_dir
endif else begin
 synop_images=chklog('$SYNOP_DATA/images')
 if not test_dir(synop_images) then return
 if verbose then message,'Copying to '+synop_images,/cont
endelse

dstart=get_def_times(tstart,tend,dend=dend,/utc)

;-- determine copy times

if is_number(back) then tback=back > 0 else tback=0
if tback gt 0 then begin
 dstart=dend
 dstart.mjd=dend.mjd-tback-1
endif

dstart=anytim2utc(dstart,/vms)
dend=anytim2utc(dend,/vms)
dprint,'% dstart,dend: ',dstart,', ',dend

if not keyword_set(all_day) then begin

;-- find overlapping GOES events

 rd_gev,dstart,dend,gev,status=status,/nearest
 if status gt 0 then begin
  message,'No GOES events during specified period',/cont
  return
 endif
 decode_gev,gev,gstart,gend,gpeak,/vms,class=class

;--- remove B flares

 flares=where(stregex(class,'(C|M|X)',/bool),nflares)

 if nflares eq 0 then begin
  message,'No flares above B level',/cont
  return
 endif

 gev=gev[flares] & gstart=gstart[flares] & gend=gend[flares]
 gpeak=gpeak[flares] & class=class[flares]

 if verbose then begin
  message,'Found following GOES events - '+arr2str(class,delim=','),/cont
 endif

;-- include preflare and postflare files

 if is_number(preflare) then pre=float(preflare) > 0. else pre=5.
 gstart=anytim2tai(gstart)-pre*60.
 gstart=anytim2utc(gstart,/vms)

;-- if M or X flare, force postflare = 30 mins

 if is_number(postflare) then post=float(postflare) > 0. else post=10.
 m_or_x=where(stregex(class,'(M|X)',/bool),fcount)
 if fcount gt 0 then post=30.

 gend=anytim2tai(gend)+post*60.
 gend=anytim2utc(gend,/vms)

endif else begin
 gstart=dstart
 gend=dend
endelse

;-- find overlapping EIT files. Search Level Zero, then QL

nflares=n_elements(gstart)
for i=0,nflares-1 do begin
 ez_files=eit_files(gstart[i],gend[i],/lz,/quiet)
 ql_files=eit_files(gstart[i],gend[i],/quick,/quiet)
 efiles=trim2([ez_files,ql_files])
 keep=where(efiles ne '',kcount)
 if kcount gt 0 then files=append_arr(files,efiles[keep],/no_copy)
endfor

if is_blank(files) then begin
 message,'No EIT files found',/cont
 return
endif

files=get_uniq(files)

;-- construct output filename, check if it exists. If not (or clobber is set),
;   prep a new one.

nfiles=n_elements(files)
eit=obj_new('eit')
eit->set,verbose=verbose

count=0
for i=0,nfiles-1 do begin
 if verbose then message,'Checking '+files[i],/cont
 eit->read,files[i],index=index,/nodata
 ndata=n_elements(index)

 for k=0,ndata-1 do begin
  chk=stregex(index[k].object,'(dark|calibration|readout|continous|lamp)',/bool,/fold)
  if chk then begin
   if verbose then message,'Skipping engineering image',/cont
   continue
  endif

  oname=eit->get_name(index[k],ymd=ymd)
  if is_string(oname) then begin
   mk_sub_dir,synop_images,ymd,out_dir=sdir
   zname=oname+'.gz'
   oname=concat_dir(sdir,oname)
   zname=concat_dir(sdir,zname)
   prep=1b 

   if not clobber then begin
    chk1=loc_file(zname,count=zcount)
    chk2=loc_file(oname,count=ocount)
    prep=(zcount eq 0) and (ocount eq 0)
   endif

   if prep then begin
    count=count+1

    if k eq 0 then eit->read,files[i]

;-- normalize to earth orbit

    eit->earth_correct,k

;-- write file

    eit->write,oname,k
   
   endif
  
   chk=loc_file(oname,count=ocount)
   zip=ocount gt 0
   if zip then zip_files=append_arr(zip_files,oname,/no_copy) 

  endif

 endfor
endfor

obj_destroy,eit

;-- gzip new files

if is_string(zip_files) then begin
 chmod,zipfiles,/g_write,/g_read
 if verbose then message,'Compressing files',/cont
 espawn,'gzip -f '+zip_files,/background,/unique
endif

message,'Processed '+trim(count)+' files',/cont


return
end


